/* xkeycaps, Copyright (c) 1991, 1992, 1993 Jamie Zawinski <jwz@jwz.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 */


#ifndef _KbdWidget_H_
#define _KbdWidget_H_

typedef struct _KeyboardRec *KeyboardWidget;
typedef struct _KeyboardClassRec *KeyboardWidgetClass;

extern WidgetClass keyboardWidgetClass;

#endif /* _KbdWidget_H_ */
