/* xkeycaps, Copyright (c) 1994, 1997 Jamie Zawinski <jwz@jwz.org>
 *
 * file for the keycodes of the GVC Win '95 keyboard (I think they use it for
 * all models) in the U.S. sold as ARM notebooks (model ARM 753CD was used
 * for creating these files)
 *
 * Note: the keycodes generated by Fn+.. keys are not included, since
 * xkeycaps does not allow two different keycodes for one "physical"
 * key. Even though it is quite common on laptops to "map" the numeric
 * keypad, etc. to other keys (wich can be accessed with a special `Fn` key)
 *
 * by R.Back <back@picard.mathematik.uni-wuerzburg.de> */

static const KeyCode gvc_codes [] = {
 /* Row 0 */
 9, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 95, 96, 77, 111, 78, 110,

 /* Row 2 */
 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 49,

 /* Row 3 */
 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 51, 97,

 /* Row 4 */
 66, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 36, 106,

 /* Row 5 */
  50, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 98, 107,

 /* Row 6 */
 0, 37, 64, 115, 94, 65, 117, 113, 109, 100, 104, 102
};
