/* xkeycaps, Copyright (c) 1994 Jamie Zawinski <jwz@jwz.org>
 *
 * This file describes the default keymap of the Takaoka Xmint X terminal.
 * Generated by Christian Goetze <c-goetze@u-aizu.ac.jp>.
 */

static const struct default_keycode_map xmint_map [] = {
  {   8, ShiftMask,	{ XK_Shift_L }},
  {   9, ShiftMask,	{ XK_Shift_R }},
  {  10, ControlMask,	{ XK_Control_L }},
  {  11, 0,		{ XK_Tab }},
  {  12, Mod1Mask,	{ XK_Meta_L }},
  {  13, Mod1Mask,	{ XK_Meta_R }},
  {  14, 0,		{ XK_Alt_L }},
  {  15, 0,		{ XK_Alt_R }},
  {  16, LockMask,	{ XK_Caps_Lock }},
  {  17, Mod5Mask,	{ XK_Mode_switch,	0,
			XK_Mode_switch }},
  {  18, 0,		{ XK_Linefeed }},
  {  19, 0,		{ XK_space }},
  {  20, 0,		{ XK_Escape }},
  {  21, 0,		{ XK_BackSpace }},
  {  22, 0,		{ XK_Delete }},
  {  23, 0,		{ XK_Return }},
  {  24, 0,		{ XK_F1 }},
  {  25, 0,		{ XK_F2 }},
  {  26, 0,		{ XK_F3 }},
  {  27, 0,		{ XK_F4 }},
  {  28, 0,		{ XK_F5 }},
  {  29, 0,		{ XK_F6 }},
  {  30, 0,		{ XK_F7 }},
  {  31, 0,		{ XK_F8 }},
  {  32, 0,		{ XK_F9 }},
  {  33, 0,		{ XK_F10 }},
  {  34, 0,		{ XK_Left,	XK_F11 }},
  {  35, 0,		{ XK_Down,	XK_F12 }},
  {  36, 0,		{ XK_Up,	XK_F13 }},
  {  37, 0,		{ XK_Right,	XK_F14 }},
  {  40, 0,		{ XK_1,	XK_exclam,
			XK_kana_NU }},
  {  41, 0,		{ XK_2,	XK_at,
			XK_kana_FU }},
  {  42, 0,		{ XK_3,	XK_numbersign,
			XK_kana_A,	XK_kana_a }},
  {  43, 0,		{ XK_4,	XK_dollar,
			XK_kana_U,	XK_kana_u }},
  {  44, 0,		{ XK_5,	XK_percent,
			XK_kana_E,	XK_kana_e }},
  {  45, 0,		{ XK_6,	XK_asciicircum,
			XK_kana_O,	XK_kana_o }},
  {  46, 0,		{ XK_7,	XK_ampersand,
			XK_kana_YA,	XK_kana_ya }},
  {  47, 0,		{ XK_8,	XK_asterisk,
			XK_kana_YU,	XK_kana_yu }},
  {  48, 0,		{ XK_9,	XK_parenleft,
			XK_kana_YO,	XK_kana_yo }},
  {  49, 0,		{ XK_0,	XK_parenright,
			XK_kana_WA,	XK_kana_WO }},
  {  50, 0,		{ XK_minus,	XK_underscore,
			XK_kana_HO }},
  {  51, 0,		{ XK_equal,	XK_plus,
			XK_kana_HE }},
  {  52, 0,		{ XK_backslash,	XK_bar,
			XK_kana_openingbracket,	XK_kana_closingbracket }},
  {  53, 0,		{ XK_grave,	XK_asciitilde,
			XK_prolongedsound }},
  {  56, 0,		{ XK_q,	XK_Q,
			XK_kana_TA }},
  {  57, 0,		{ XK_w,	XK_W,
			XK_kana_TE }},
  {  58, 0,		{ XK_e,	XK_E,
			XK_kana_I,	XK_kana_i }},
  {  59, 0,		{ XK_r,	XK_R,
			XK_kana_SU }},
  {  60, 0,		{ XK_t,	XK_T,
			XK_kana_KA }},
  {  61, 0,		{ XK_y,	XK_Y,
			XK_kana_N }},
  {  62, 0,		{ XK_u,	XK_U,
			XK_kana_NA }},
  {  63, 0,		{ XK_i,	XK_I,
			XK_kana_NI }},
  {  64, 0,		{ XK_o,	XK_O,
			XK_kana_RA }},
  {  65, 0,		{ XK_p,	XK_P,
			XK_kana_SE }},
  {  66, 0,		{ XK_bracketleft,	XK_braceleft,
			XK_kana_RO,	XK_voicedsound }},
  {  67, 0,		{ XK_bracketright,	XK_braceright,
			XK_kana_MU,	XK_semivoicedsound }},
  {  72, 0,		{ XK_a,	XK_A,
			XK_kana_CHI }},
  {  73, 0,		{ XK_s,	XK_S,
			XK_kana_TO }},
  {  74, 0,		{ XK_d,	XK_D,
			XK_kana_SHI }},
  {  75, 0,		{ XK_f,	XK_F,
			XK_kana_HA }},
  {  76, 0,		{ XK_g,	XK_G,
			XK_kana_KI }},
  {  77, 0,		{ XK_h,	XK_H,
			XK_kana_KU }},
  {  78, 0,		{ XK_j,	XK_J,
			XK_kana_MA }},
  {  79, 0,		{ XK_k,	XK_K,
			XK_kana_NO }},
  {  80, 0,		{ XK_l,	XK_L,
			XK_kana_RI }},
  {  81, 0,		{ XK_semicolon,	XK_colon,
			XK_kana_RE }},
  {  82, 0,		{ XK_apostrophe,	XK_quotedbl,
			XK_kana_KE }},
  {  88, 0,		{ XK_z,	XK_Z,
			XK_kana_TSU,	XK_kana_tsu }},
  {  89, 0,		{ XK_x,	XK_X,
			XK_kana_SA }},
  {  90, 0,		{ XK_c,	XK_C,
			XK_kana_SO }},
  {  91, 0,		{ XK_v,	XK_V,
			XK_kana_HI }},
  {  92, 0,		{ XK_b,	XK_B,
			XK_kana_KO }},
  {  93, 0,		{ XK_n,	XK_N,
			XK_kana_MI }},
  {  94, 0,		{ XK_m,	XK_M,
			XK_kana_MO }},
  {  95, 0,		{ XK_comma,	XK_less,
			XK_kana_NE,	XK_kana_comma }},
  {  96, 0,		{ XK_period,	XK_greater,
			XK_kana_RU,	XK_kana_fullstop }},
  {  97, 0,		{ XK_slash,	XK_question,
			XK_kana_ME,	XK_kana_conjunctive }}
  };
