/* xkeycaps, Copyright (c) 1999 Jamie Zawinski <jwz@jwz.org>
 *
 * This file describes the key surfaces of the DEC LK450 keyboard.
 * It uses conditionalisations in lk201-caps.h, since the keyboards are
 * physically quite similar.
 *
 * Robin Stephenson <robin.stephenson@blackwell.co.uk> 1998
 *
 */

#define LK450_KLUDGE
#include "lk201-caps.h"
#undef LK450_KLUDGE
